@@ -167,6 +167,20 @@ class Config(object):
             self.roles = [role.strip() for role in args.roles.split(',') if role.strip()]
         self.conf_profile = args.profile or 'DEFAULT'
 
+    def _handle_config(self, config, profile_config):
+        if "inherits" in profile_config.keys():
+            print("Using inherited config: " + profile_config["inherits"])
+            if profile_config["inherits"] not in config:
+                raise errors.GimmeAWSCredsError(self.conf_profile + " inherits from " + profile_config["inherits"] + ", but could not find " + profile_config["inherits"])
+            combined_config = {
+                **self._handle_config(config, dict(config[profile_config["inherits"]])),
+                **profile_config,
+            }
+            del combined_config["inherits"]
+            return combined_config
+        else:
+            return profile_config
+
     def get_config_dict(self):
         """returns the conf dict from the okta config file"""
         # Check to see if config file exists, if not complain and exit
@@ -177,19 +191,7 @@ class Config(object):
 
             try:
                 profile_config = dict(config[self.conf_profile])
-                if "inherits" in profile_config.keys():
-                    print("Using inherited config: " + profile_config["inherits"])
-                    if profile_config["inherits"] not in config:
-                        raise errors.GimmeAWSCredsError(self.conf_profile + " inherits from " + profile_config["inherits"] + ", but could not find " + profile_config["inherits"])
-                    combined_config = {
-                        **dict(config[profile_config["inherits"]]),
-                        **profile_config,
-                    }
-                    del combined_config["inherits"]
-                    return combined_config
-                else:
-                    return profile_config
-
+                return self._handle_config(config, profile_config)
             except KeyError:
                 raise errors.GimmeAWSCredsError(
                     'Configuration profile not found! Use the --action-configure flag to generate the profile.')
@@ -477,14 +479,24 @@ class Config(object):
         """Get the user's preferred MFA device [Optional]"""
         ui.default.message(
             "If you'd like to set a preferred device type to use for MFA, enter it here.\n"
-            "This is optional. valid devices types:[sms, call, push, token, token:software:totp]")
+            "This is optional. valid devices types:\n"
+            """
+            - push - Okta Verify App push or DUO push (depends on okta supplied provider type)
+            - token:software:totp - OTP using the Okta Verify App
+            - token:hardware - OTP using hardware like Yubikey
+            - call - OTP via Voice call
+            - sms - OTP via SMS message
+            - web - DUO uses localhost webbrowser to support push|call|passcode
+            - passcode - DUO uses `OKTA_MFA_CODE` or `--mfa-code` if set, or prompts user for passcode(OTP).
+            """
+        )
         okta_username = self._get_user_input(
             "Preferred MFA Device Type", default_entry)
         return okta_username
 
     def _get_output_format(self, default_entry):
         """Get the user's preferred output format [Optional]"""
-        ui.default.message("Set the tools' output format:[bash, json]")
+        ui.default.message("Set the tools' output format:[export, json]")
         output_format = None
         while output_format not in ('export', 'json'):
             output_format = self._get_user_input(
